module ALU(
    Operand0,
    Operand1,
    Opcode,

    Result,
    Overflow,

    Branch
);

parameter Width = 32;

`include "ControlConstants.v"

localparam OpcodeWidth = 5;

input       [Width-1:0]         Operand0; 
input       [Width-1:0]         Operand1;

input       [OpcodeWidth-1:0]   Opcode;

output      [Width-1:0]         Result;
output                          Overflow;

output                          Branch;

wire signed [Width-1:0]         sOperand0;
wire signed [Width-1:0]         sOperand1;

wire        [Width:0]           add;
wire        [Width:0]           sub;
wire        [Width-1:0]         sll;
wire        [Width-1:0]         srl;
wire        [Width-1:0]         sra;

reg         [Width:0]           result;
reg                             branch;

assign Result = result[Width-1:0];
assign Overflow = result[Width:Width];

assign Branch = branch;

assign sOperand0 = Operand0;
assign sOperand1 = Operand1;

assign add = Operand0 + Operand1;
assign sub = Operand0 - Operand1;
assign sll = Operand1 << Operand0[4:0];
assign srl = Operand1 >> Operand0[4:0];
assign sra = sOperand1 >>> Operand0[4:0];

always@(*) begin
    branch = 1'bx;
    result = {Width{1'bx}};
    case(Opcode)
        ALUADD:   result = add;
        ALUADDU:  result = {1'b0, add[Width-1:0]};
        ALUSLT:   result = {{Width-1{1'b0}}, sOperand0 < sOperand1};
        ALUSLTU:  result = {{Width-1{1'b0}}, Operand0 < Operand1};
        ALUAND:   result = {1'b0, Operand0 & Operand1};
        ALUOR:    result = {1'b0, Operand0 | Operand1};
        ALUXOR:   result = {1'b0, Operand0 ^ Operand1};
        ALULUI:   result = {1'b0, Operand1[(Width/2)-1:0], {(Width/2){1'b0}}};
        ALUSLL:   result = {1'b0, sll[Width-1:0]};
        ALUSRL:   result = {1'b0, srl[Width-1:0]};
        ALUSRA:   result = {1'b0, sra[Width-1:0]};
        ALUSUB:   result = sub;
        ALUSUBU:  result = {1'b0, sub[Width-1:0]};
        ALUNOR:   result = {1'b0, ~(Operand0 | Operand1)};
        ALUPASS0: result = Operand0;
        ALUPASS1: result = Operand1;
        ALUBEQ:   branch = Operand0 == Operand1;
        ALUBNE:   branch = Operand0 != Operand1;
        ALUBLEZ:  branch = Operand0 <= {Width{1'b0}};
        ALUBGTZ:  branch = Operand0 >  {Width{1'b0}};
        ALUBLTZ:  branch = Operand0 <  {Width{1'b0}};
        ALUBGEZ:  branch = Operand0 >= {Width{1'b0}};
    endcase
end

endmodule
