module DataMemory(
    Clock,
    Enable,

    DataAddress,
    DataIn,
    DataInEnable,
    DataOut,
    DataOutEnable
);

parameter DataWidth = 32,
          AddressWidth = 12,
          InitialHexFile = "";

localparam Depth = 2 ** (AddressWidth-2);


input                           Clock;
input                           Enable;

input       [AddressWidth-1:0]  DataAddress;
input       [DataWidth-1:0]     DataIn;
input                           DataInEnable;
output      [DataWidth-1:0]     DataOut;
input                           DataOutEnable;


reg         [AddressWidth-3:0]  address;

reg         [DataWidth-1:0]     state[Depth-1:0];


generate if(InitialHexFile != "")
initial
    $readmemh(InitialHexFile, state);
endgenerate


assign DataInReady  = 1'b1;

assign DataOut      = state[address];


always@(posedge Clock) begin
    if(Enable) begin
        if(DataOutEnable) begin
            address <= DataAddress[AddressWidth-1:2];
        end

        if(DataInEnable) begin
            state[DataAddress[AddressWidth-1:2]] <= DataIn;
        end
    end
end

endmodule
